@@ -1426,7 +1426,6 @@ static int connect_stream(struct file *file)
 {
 	int res;
 	struct global *g = file->node;
-	const char *str;
 	struct timespec abstime;
 	const char *error = NULL;
 	uint8_t buffer[1024];
@@ -1442,11 +1441,7 @@ static int connect_stream(struct file *file)
 
 	disconnect_stream(file);
 
-	props = NULL;
-	if ((str = getenv("PIPEWIRE_PROPS")) != NULL)
-		props = pw_properties_new_string(str);
-	if (props == NULL)
-		props = pw_properties_new(NULL, NULL);
+	props = pw_properties_new(NULL, NULL);
 	if (props == NULL) {
 		res = -errno;
 		goto exit;
@@ -1470,6 +1465,7 @@ static int connect_stream(struct file *file)
 			&file->stream_listener,
 			&stream_events, file);
 
+
 	file->error = 0;
 
 	pw_stream_connect(file->stream,
@@ -1760,7 +1756,7 @@ static int v4l2_ioctl(int fd, unsigned long int request, void *arg)
 	if ((file = find_file(fd)) == NULL)
 		return globals.old_fops.ioctl(fd, request, arg);
 
-#ifdef __FreeBSD__
+#if defined(__FreeBSD__) || defined(__MidnightBSD__)
 	if (arg == NULL && (request & IOC_DIRMASK != IOC_VOID)) {
 #else
 	if (arg == NULL && (_IOC_DIR(request) & (_IOC_WRITE | _IOC_READ))) {
