@@ -37,9 +37,8 @@
 #include <spa/support/system.h>
 #include <spa/control/control.h>
 
-#define NAME "alsa-seq"
+#include "alsa.h"
 
-#include "dll.h"
 #include "alsa-seq.h"
 
 #define CHECK(s,msg,...) if ((res = (s)) < 0) { spa_log_error(state->log, msg ": %s", ##__VA_ARGS__, snd_strerror(res)); return res; }
@@ -132,13 +131,19 @@ static int seq_close(struct seq_state *state, struct seq_conn *conn)
 static int init_stream(struct seq_state *state, enum spa_direction direction)
 {
 	struct seq_stream *stream = &state->streams[direction];
+	int res;
 	stream->direction = direction;
 	if (direction == SPA_DIRECTION_INPUT) {
 		stream->caps = SND_SEQ_PORT_CAP_SUBS_WRITE;
 	} else {
 		stream->caps = SND_SEQ_PORT_CAP_SUBS_READ;
 	}
-	snd_midi_event_new(MAX_EVENT_SIZE, &stream->codec);
+	if ((res = snd_midi_event_new(MAX_EVENT_SIZE, &stream->codec)) < 0) {
+		spa_log_error(state->log, "can make event decoder: %s",
+				snd_strerror(res));
+		return res;
+	}
+	snd_midi_event_no_status(stream->codec, 1);
 	memset(stream->ports, 0, sizeof(stream->ports));
 	return 0;
 }
@@ -146,7 +151,9 @@ static int init_stream(struct seq_state *state, enum spa_direction direction)
 static int uninit_stream(struct seq_state *state, enum spa_direction direction)
 {
 	struct seq_stream *stream = &state->streams[direction];
-	snd_midi_event_free(stream->codec);
+	if (stream->codec)
+		snd_midi_event_free(stream->codec);
+	stream->codec = NULL;
 	return 0;
 }
 
@@ -179,7 +186,7 @@ static void init_ports(struct seq_state *state)
 
 static void debug_event(struct seq_state *state, snd_seq_event_t *ev)
 {
-	if (SPA_LIKELY(!spa_log_level_enabled(state->log, SPA_LOG_LEVEL_TRACE)))
+	if (SPA_LIKELY(!spa_log_level_topic_enabled(state->log, SPA_LOG_TOPIC_DEFAULT, SPA_LOG_LEVEL_TRACE)))
 		return;
 
 	spa_log_trace(state->log, "event type:%d flags:0x%x", ev->type, ev->flags);
@@ -247,6 +254,11 @@ static void alsa_seq_on_sys(struct spa_source *source)
 					addr->client, addr->port);
 			state->port_info(state->port_info_data, addr, NULL);
 			break;
+		default:
+			spa_log_info(state->log, "unhandled event %d: %d:%d",
+					ev->type, addr->client, addr->port);
+			break;
+
 		}
 		snd_seq_free_event(ev);
         }
@@ -453,7 +465,7 @@ int spa_alsa_seq_recycle_buffer(struct seq_state *state, struct seq_port *port,
 	struct buffer *b = &port->buffers[buffer_id];
 
 	if (SPA_FLAG_IS_SET(b->flags, BUFFER_FLAG_OUT)) {
-		spa_log_trace_fp(state->log, NAME " %p: recycle buffer port:%p buffer-id:%u",
+		spa_log_trace_fp(state->log, "%p: recycle buffer port:%p buffer-id:%u",
 				state, port, buffer_id);
 		spa_list_append(&port->free, &b->link);
 		SPA_FLAG_CLEAR(b->flags, BUFFER_FLAG_OUT);
@@ -498,6 +510,9 @@ static int process_recycle(struct seq_state *state)
 	return 0;
 }
 
+#define NSEC_TO_CLOCK(r,n) (((n) * (r)->denom) / ((r)->num * SPA_NSEC_PER_SEC))
+#define NSEC_FROM_CLOCK(r,n) (((n) * (r)->num * SPA_NSEC_PER_SEC) / (r)->denom)
+
 static int process_read(struct seq_state *state)
 {
 	snd_seq_event_t *ev;
@@ -536,12 +551,6 @@ static int process_read(struct seq_state *state)
 			continue;
 		}
 
-		/* fixup NoteOn with vel 0 */
-		if ((data[0] & 0xF0) == 0x90 && data[2] == 0x00) {
-			data[0] = 0x80 + (data[0] & 0x0F);
-			data[2] = 0x40;
-		}
-
 		/* queue_time is the estimated current time of the queue as calculated by
 		 * the DLL. Calculate the age of the event. */
 		ev_time = SPA_TIMESPEC_TO_NSEC(&ev->time.time);
@@ -551,9 +560,9 @@ static int process_read(struct seq_state *state)
 			diff = 0;
 
 		/* convert the age to samples and convert to an offset */
-		offset = (diff * state->rate.denom) / (state->rate.num * SPA_NSEC_PER_SEC);
+		offset = NSEC_TO_CLOCK(&state->rate, diff);
 		if (state->duration > offset)
-			offset = state->duration - offset;
+			offset = state->duration - 1 - offset;
 		else
 			offset = 0;
 
@@ -619,7 +628,7 @@ static int process_write(struct seq_state *state)
 {
 	struct seq_stream *stream = &state->streams[SPA_DIRECTION_INPUT];
 	uint32_t i;
-	int res = 0;
+	int err, res = 0;
 
 	for (i = 0; i < stream->last_port; i++) {
 		struct seq_port *port = &stream->ports[i];
@@ -673,8 +682,7 @@ static int process_write(struct seq_state *state)
 			snd_seq_ev_set_source(&ev, state->event.addr.port);
 			snd_seq_ev_set_dest(&ev, port->addr.client, port->addr.port);
 
-			out_time = state->queue_time +
-				(c->offset * state->rate.num * SPA_NSEC_PER_SEC) / state->rate.denom;
+			out_time = state->queue_time + NSEC_FROM_CLOCK(&state->rate, c->offset);
 
 			out_rt.tv_nsec = out_time % SPA_NSEC_PER_SEC;
 			out_rt.tv_sec = out_time / SPA_NSEC_PER_SEC;
@@ -683,9 +691,9 @@ static int process_write(struct seq_state *state)
 			spa_log_trace_fp(state->log, "event time:%"PRIu64" offset:%d size:%ld port:%d.%d",
 				out_time, c->offset, size, port->addr.client, port->addr.port);
 
-			if ((res = snd_seq_event_output(state->event.hndl, &ev)) < 0) {
+			if ((err = snd_seq_event_output(state->event.hndl, &ev)) < 0) {
 				spa_log_warn(state->log, "failed to output event: %s",
-						snd_strerror(res));
+						snd_strerror(err));
 			}
 		}
 	}
@@ -694,25 +702,25 @@ static int process_write(struct seq_state *state)
 	return res;
 }
 
-#define NSEC_TO_CLOCK(c,n) ((n) * (c)->rate.denom / ((c)->rate.num * SPA_NSEC_PER_SEC))
-
 static int update_time(struct seq_state *state, uint64_t nsec, bool follower)
 {
 	snd_seq_queue_status_t *status;
 	const snd_seq_real_time_t* queue_time;
-	uint64_t queue_real, position;
+	uint64_t queue_real;
 	double err, corr;
-	uint64_t clock_elapsed, queue_elapsed;
+	uint64_t queue_elapsed;
 
 	if (state->position) {
 		struct spa_io_clock *clock = &state->position->clock;
 		state->rate = clock->rate;
 		state->duration = clock->duration;
-		state->threshold = state->duration;
-		position = clock->position;
 	} else {
-		position = 0;
+		state->rate = SPA_FRACTION(1, 48000);
+		state->duration = 1024;
 	}
+	state->threshold = state->duration;
+
+	corr = 1.0 - (state->dll.z2 + state->dll.z3);
 
 	/* take queue time */
 	snd_seq_queue_status_alloca(&status);
@@ -720,18 +728,17 @@ static int update_time(struct seq_state *state, uint64_t nsec, bool follower)
 	queue_time = snd_seq_queue_status_get_real_time(status);
 	queue_real = SPA_TIMESPEC_TO_NSEC(queue_time);
 
-	if (state->queue_base == 0) {
-		state->queue_base = nsec - queue_real;
-		state->clock_base = position;
-	}
+	if (state->queue_time == 0)
+		queue_elapsed = 0;
+	else
+		queue_elapsed = (queue_real - state->queue_time) / corr;
 
-	corr = 1.0 - (state->dll.z2 + state->dll.z3);
+	state->queue_time = queue_real;
 
-	clock_elapsed = position - state->clock_base;
-	state->queue_time = nsec - state->queue_base;
-	queue_elapsed = NSEC_TO_CLOCK(state->clock, state->queue_time) / corr;
+	queue_elapsed = NSEC_TO_CLOCK(&state->rate, queue_elapsed);
 
-	err = ((int64_t)clock_elapsed - (int64_t) queue_elapsed);
+	err = ((int64_t)state->threshold - (int64_t) queue_elapsed);
+	err = SPA_CLAMP(err, -64, 64);
 
 	if (state->dll.bw == 0.0) {
 		spa_dll_set_bw(&state->dll, SPA_DLL_BW_MAX, state->threshold,
@@ -743,7 +750,7 @@ static int update_time(struct seq_state *state, uint64_t nsec, bool follower)
 
 	if ((state->next_time - state->base_time) > BW_PERIOD) {
 		state->base_time = state->next_time;
-		spa_log_debug(state->log, NAME" %p: follower:%d rate:%f bw:%f err:%f (%f %f %f)",
+		spa_log_debug(state->log, "%p: follower:%d rate:%f bw:%f err:%f (%f %f %f)",
 				state, follower, corr, state->dll.bw, err,
 				state->dll.z1, state->dll.z2, state->dll.z3);
 	}
@@ -834,8 +841,10 @@ static void reset_stream(struct seq_state *this, struct seq_stream *stream, bool
 static int set_timers(struct seq_state *state)
 {
 	struct timespec now;
+	int res;
 
-	spa_system_clock_gettime(state->data_system, CLOCK_MONOTONIC, &now);
+	if ((res = spa_system_clock_gettime(state->data_system, CLOCK_MONOTONIC, &now)) < 0)
+	    return res;
 
 	state->next_time = SPA_TIMESPEC_TO_NSEC(&now);
 	if (state->following) {
@@ -891,7 +900,7 @@ int spa_alsa_seq_start(struct seq_state *state)
 	state->source.rmask = 0;
 	spa_loop_add_source(state->data_loop, &state->source);
 
-	state->queue_base = 0;
+	state->queue_time = 0;
 	spa_dll_init(&state->dll);
 	set_timers(state);
 
