@@ -30,7 +30,7 @@
 #include <sched.h>
 #include <fcntl.h>
 
-#ifdef __FreeBSD__
+#if defined(__FreeBSD__) || defined(__MidnightBSD__)
 #include <sys/sysctl.h>
 #endif
 
@@ -40,8 +40,11 @@
 #include <spa/utils/type.h>
 #include <spa/utils/hook.h>
 #include <spa/utils/names.h>
+#include <spa/utils/string.h>
 
-#define NAME "cpu"
+static struct spa_log_topic log_topic = SPA_LOG_TOPIC(0, "spa.cpu");
+#undef SPA_LOG_TOPIC_DEFAULT
+#define SPA_LOG_TOPIC_DEFAULT &log_topic
 
 struct impl {
 	struct spa_handle handle;
@@ -75,11 +78,14 @@ static char *read_file(const char *name, char *buffer, size_t len)
 # if defined (__i386__) || defined (__x86_64__)
 #include "cpu-x86.c"
 #define init(t)	x86_init(t)
+#define impl_cpu_zero_denormals x86_zero_denormals
 # elif defined (__arm__) || defined (__aarch64__)
 #include "cpu-arm.c"
 #define init(t)	arm_init(t)
+#define impl_cpu_zero_denormals arm_zero_denormals
 # else
 #define init(t)
+#define impl_cpu_zero_denormals NULL
 #endif
 
 static uint32_t
@@ -111,7 +117,7 @@ static uint32_t get_count(struct impl *this)
 #else
 static uint32_t get_count(struct impl *this)
 {
-	int mib[] = {CTL_HW, HW_NCPU};
+	static const int mib[] = {CTL_HW, HW_NCPU};
 	int r;
 	size_t rSize = sizeof(r);
 	if(-1 == sysctl(mib, 2, &r, &rSize, 0, 0))
@@ -174,7 +180,7 @@ impl_cpu_get_vm_type(void *object)
 			continue;
 
 		for (j = 0; j < SPA_N_ELEMENTS(dmi_vendor_table); j++) {
-			if (strstr(s, dmi_vendor_table[j].vendor) == s) {
+			if (spa_strstartswith(s, dmi_vendor_table[j].vendor)) {
 				spa_log_debug(impl->log, "Virtualization %s found in DMI (%s)",
 						s, dmi_vendors[i]);
 				impl->vm_type = dmi_vendor_table[j].id;
@@ -194,6 +200,7 @@ static const struct spa_cpu_methods impl_cpu = {
 	.get_count = impl_cpu_get_count,
 	.get_max_align = impl_cpu_get_max_align,
 	.get_vm_type = impl_cpu_get_vm_type,
+	.zero_denormals = impl_cpu_zero_denormals,
 };
 
 static int impl_get_interface(struct spa_handle *handle, const char *type, void **interface)
@@ -205,7 +212,7 @@ static int impl_get_interface(struct spa_handle *handle, const char *type, void
 
 	this = (struct impl *) handle;
 
-	if (strcmp(type, SPA_TYPE_INTERFACE_CPU) == 0)
+	if (spa_streq(type, SPA_TYPE_INTERFACE_CPU))
 		*interface = &this->cpu;
 	else
 		return -ENOENT;
@@ -249,6 +256,7 @@ impl_init(const struct spa_handle_factory *factory,
 			&impl_cpu, this);
 
 	this->log = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_Log);
+	spa_log_topic_init(this->log, &log_topic);
 
 	this->flags = 0;
 	this->force = SPA_CPU_FORCE_AUTODETECT;
@@ -261,9 +269,11 @@ impl_init(const struct spa_handle_factory *factory,
 			this->flags = atoi(str);
 		if ((str = spa_dict_lookup(info, SPA_KEY_CPU_VM_TYPE)) != NULL)
 			this->vm_type = atoi(str);
+		if ((str = spa_dict_lookup(info, SPA_KEY_CPU_ZERO_DENORMALS)) != NULL)
+			spa_cpu_zero_denormals(&this->cpu, spa_atob(str));
 	}
 
-	spa_log_debug(this->log, NAME " %p: count:%d align:%d flags:%08x",
+	spa_log_debug(this->log, "%p: count:%d align:%d flags:%08x",
 			this, this->count, this->max_align, this->flags);
 
 	return 0;
