@@ -140,8 +140,13 @@ uint32_t pw_protocol_native_connection_add_fd(struct pw_protocol_native_connecti
 	}
 
 	buf->msg.fds[index] = fcntl(fd, F_DUPFD_CLOEXEC, 0);
+	if (buf->msg.fds[index] == -1) {
+		pw_log_error("connection %p: can't DUP fd:%d %m", conn, fd);
+		return SPA_IDX_INVALID;
+	}
 	buf->msg.n_fds++;
-	pw_log_debug("connection %p: add fd %d at index %d", conn, fd, index);
+	pw_log_debug("connection %p: add fd %d (new fd:%d) at index %d",
+			conn, fd, buf->msg.fds[index], index);
 
 	return index;
 }
@@ -151,17 +156,23 @@ static void *connection_ensure_size(struct pw_protocol_native_connection *conn,
 	int res;
 
 	if (buf->buffer_size + size > buf->buffer_maxsize) {
-		buf->buffer_maxsize = SPA_ROUND_UP_N(buf->buffer_size + size, MAX_BUFFER_SIZE);
-		buf->buffer_data = realloc(buf->buffer_data, buf->buffer_maxsize);
-		if (buf->buffer_data == NULL) {
+		void *np;
+		size_t ns;
+
+		ns = SPA_ROUND_UP_N(buf->buffer_size + size, MAX_BUFFER_SIZE);
+		np = realloc(buf->buffer_data, ns);
+		if (np == NULL) {
 			res = -errno;
+			free(buf->buffer_data);
 			buf->buffer_maxsize = 0;
 			spa_hook_list_call(&conn->listener_list,
 					struct pw_protocol_native_connection_events,
-					error, 0, -res);
+					error, 0, res);
 			errno = -res;
 			return NULL;
 		}
+		buf->buffer_maxsize = ns;
+		buf->buffer_data = np;
 		pw_log_debug("connection %p: resize buffer to %zd %zd %zd",
 			    conn, buf->buffer_size, size, buf->buffer_maxsize);
 	}
@@ -198,6 +209,7 @@ static void close_all_fds(struct msghdr *msg, struct cmsghdr *from)
 			int fd;
 
 			memcpy(&fd, p, sizeof(fd));
+			pw_log_debug("%p: close fd:%d", msg, fd);
 			close(fd);
 		}
 	}
@@ -209,7 +221,10 @@ static int refill_buffer(struct pw_protocol_native_connection *conn, struct buff
 	struct cmsghdr *cmsg = NULL;
 	struct msghdr msg = { 0 };
 	struct iovec iov[1];
-	char cmsgbuf[CMSG_SPACE(MAX_FDS_MSG * sizeof(int))];
+	union {
+		char cmsgbuf[CMSG_SPACE(MAX_FDS_MSG * sizeof(int))];
+		struct cmsghdr align;
+	} cmsgbuf;
 	int n_fds = 0;
 	size_t avail;
 
@@ -219,7 +234,7 @@ static int refill_buffer(struct pw_protocol_native_connection *conn, struct buff
 	iov[0].iov_len = avail;
 	msg.msg_iov = iov;
 	msg.msg_iovlen = 1;
-	msg.msg_control = cmsgbuf;
+	msg.msg_control = &cmsgbuf;
 	msg.msg_controllen = sizeof(cmsgbuf);
 	msg.msg_flags = MSG_CMSG_CLOEXEC | MSG_DONTWAIT;
 
@@ -275,8 +290,10 @@ static void clear_buffer(struct buffer *buf, bool fds)
 {
 	uint32_t i;
 	if (fds) {
-		for (i = 0; i < buf->n_fds; i++)
+		for (i = 0; i < buf->n_fds; i++) {
+			pw_log_debug("%p: close fd:%d", buf, buf->fds[i]);
 			close(buf->fds[i]);
+		}
 	}
 	buf->n_fds = 0;
 	buf->buffer_size = 0;
@@ -741,7 +758,10 @@ int pw_protocol_native_connection_flush(struct pw_protocol_native_connection *co
 	struct msghdr msg = { 0 };
 	struct iovec iov[1];
 	struct cmsghdr *cmsg;
-	char cmsgbuf[CMSG_SPACE(MAX_FDS_MSG * sizeof(int))];
+	union {
+		char cmsgbuf[CMSG_SPACE(MAX_FDS_MSG * sizeof(int))];
+		struct cmsghdr align;
+	} cmsgbuf;
 	int res = 0, *fds;
 	uint32_t fds_len, to_close, n_fds, outfds, i;
 	struct buffer *buf;
@@ -772,7 +792,7 @@ int pw_protocol_native_connection_flush(struct pw_protocol_native_connection *co
 		msg.msg_iovlen = 1;
 
 		if (outfds > 0) {
-			msg.msg_control = cmsgbuf;
+			msg.msg_control = &cmsgbuf;
 			msg.msg_controllen = CMSG_SPACE(fds_len);
 			cmsg = CMSG_FIRSTHDR(&msg);
 			cmsg->cmsg_level = SOL_SOCKET;
@@ -813,8 +833,10 @@ exit:
 	if (size > 0)
 		memmove(buf->buffer_data, data, size);
 	buf->buffer_size = size;
-	for (i = 0; i < to_close; i++)
+	for (i = 0; i < to_close; i++) {
+		pw_log_debug("%p: close fd:%d", conn, buf->fds[i]);
 		close(buf->fds[i]);
+	}
 	if (n_fds > 0)
 		memmove(buf->fds, fds, n_fds * sizeof(int));
 	buf->n_fds = n_fds;
