@@ -32,7 +32,6 @@
 
 #include "../defs.h"
 #include "../module.h"
-#include "registry.h"
 
 #define NAME "tunnel-sink"
 
@@ -82,7 +81,7 @@ static int module_tunnel_sink_load(struct client *client, struct module *module)
 	fprintf(f, "{");
 	pw_properties_serialize_dict(f, &module->props->dict, 0);
 	fprintf(f, " pulse.server.address = \"%s\" ", server);
-	fprintf(f, " tunnel.mode = playback ");
+	fprintf(f, " tunnel.mode = sink ");
 	if (data->latency_msec > 0)
 		fprintf(f, " pulse.latency = %u ", data->latency_msec);
 	fprintf(f, " stream.props = {");
@@ -120,12 +119,6 @@ static int module_tunnel_sink_unload(struct module *module)
 	return 0;
 }
 
-static const struct module_methods module_tunnel_sink_methods = {
-	VERSION_MODULE_METHODS,
-	.load = module_tunnel_sink_load,
-	.unload = module_tunnel_sink_unload,
-};
-
 static const struct spa_dict_item module_tunnel_sink_info[] = {
 	{ PW_KEY_MODULE_AUTHOR, "Wim Taymans <wim.taymans@gmail.com>" },
 	{ PW_KEY_MODULE_DESCRIPTION, "Create a network sink which connects to a remote PulseAudio server" },
@@ -156,25 +149,22 @@ static void audio_info_to_props(struct spa_audio_info_raw *info, struct pw_prope
 	pw_properties_set(props, SPA_KEY_AUDIO_POSITION, s);
 }
 
-struct module *create_module_tunnel_sink(struct impl *impl, const char *argument)
+static int module_tunnel_sink_prepare(struct module * const module)
 {
-	struct module *module;
-	struct module_tunnel_sink_data *d;
-	struct pw_properties *props = NULL, *stream_props = NULL;
+	struct module_tunnel_sink_data * const d = module->user_data;
+	struct pw_properties * const props = module->props;
+	struct pw_properties *stream_props = NULL;
 	const char *str, *server, *remote_sink_name;
 	struct spa_audio_info_raw info = { 0 };
 	int res;
 
 	PW_LOG_TOPIC_INIT(mod_topic);
 
-	props = pw_properties_new_dict(&SPA_DICT_INIT_ARRAY(module_tunnel_sink_info));
 	stream_props = pw_properties_new(NULL, NULL);
-	if (props == NULL || stream_props == NULL) {
+	if (stream_props == NULL) {
 		res = -ENOMEM;
 		goto out;
 	}
-	if (argument)
-		module_args_add_props(props, argument);
 
 	remote_sink_name = pw_properties_get(props, "sink");
 	if (remote_sink_name)
@@ -203,7 +193,7 @@ struct module *create_module_tunnel_sink(struct impl *impl, const char *argument
 		module_args_add_props(stream_props, str);
 		pw_properties_set(props, "sink_properties", NULL);
 	}
-	if (module_args_to_audioinfo(impl, props, &info) < 0) {
+	if (module_args_to_audioinfo(module->impl, props, &info) < 0) {
 		res = -EINVAL;
 		goto out;
 	}
@@ -219,23 +209,23 @@ struct module *create_module_tunnel_sink(struct impl *impl, const char *argument
 		pw_properties_set(stream_props, PW_KEY_AUDIO_FORMAT, format_id2name(id));
 	}
 
-	module = module_new(impl, &module_tunnel_sink_methods, sizeof(*d));
-	if (module == NULL) {
-		res = -errno;
-		goto out;
-	}
-
-	module->props = props;
-	d = module->user_data;
 	d->module = module;
 	d->stream_props = stream_props;
 
 	pw_properties_fetch_uint32(props, "latency_msec", &d->latency_msec);
 
-	return module;
+	return 0;
 out:
-	pw_properties_free(props);
 	pw_properties_free(stream_props);
-	errno = -res;
-	return NULL;
+
+	return res;
 }
+
+DEFINE_MODULE_INFO(module_tunnel_sink) = {
+	.name = "module-tunnel-sink",
+	.prepare = module_tunnel_sink_prepare,
+	.load = module_tunnel_sink_load,
+	.unload = module_tunnel_sink_unload,
+	.properties = &SPA_DICT_INIT_ARRAY(module_tunnel_sink_info),
+	.data_size = sizeof(struct module_tunnel_sink_data),
+};
