@@ -109,7 +109,7 @@ struct tunnel_info {
 	const char *domain;
 };
 
-#define TUNNEL_INFO(...) (struct tunnel_info){ __VA_ARGS__ }
+#define TUNNEL_INFO(...) ((struct tunnel_info){ __VA_ARGS__ })
 
 struct tunnel {
 	struct spa_list link;
@@ -195,10 +195,10 @@ static void pw_properties_from_avahi_string(const char *key, const char *value,
 		pw_properties_set(props, PW_KEY_NODE_TARGET, value);
 	}
 	else if (spa_streq(key, "rate")) {
-		pw_properties_setf(props, PW_KEY_AUDIO_RATE, "%u", atoi(value));
+		pw_properties_set(props, PW_KEY_AUDIO_RATE, value);
 	}
 	else if (spa_streq(key, "channels")) {
-		pw_properties_setf(props, PW_KEY_AUDIO_CHANNELS, "%u", atoi(value));
+		pw_properties_set(props, PW_KEY_AUDIO_CHANNELS, value);
 	}
 	else if (spa_streq(key, "channel_map")) {
 		struct channel_map channel_map;
@@ -311,7 +311,7 @@ static void resolver_cb(AvahiServiceResolver *r, AvahiIfIndex interface, AvahiPr
 		pw_properties_setf(props, PW_KEY_NODE_NAME,
 				"tunnel.%s", host_name);
 
-	str = strstr(type, "sink") ? "playback" : "capture";
+	str = strstr(type, "sink") ? "sink" : "source";
 	pw_properties_set(props, "tunnel.mode", str);
 
 	if (a->proto == AVAHI_PROTO_INET6 &&
