@@ -502,10 +502,10 @@ static const struct pw_device_methods device_methods = {
 };
 
 static int
-global_bind(void *_data, struct pw_impl_client *client, uint32_t permissions,
+global_bind(void *object, struct pw_impl_client *client, uint32_t permissions,
 		  uint32_t version, uint32_t id)
 {
-	struct pw_impl_device *this = _data;
+	struct pw_impl_device *this = object;
 	struct pw_global *global = this->global;
 	struct pw_resource *resource;
 	struct resource_data *data;
@@ -540,9 +540,9 @@ error_resource:
 	return -errno;
 }
 
-static void global_destroy(void *object)
+static void global_destroy(void *data)
 {
-	struct pw_impl_device *device = object;
+	struct pw_impl_device *device = data;
 	spa_hook_remove(&device->global_listener);
 	device->global = NULL;
 	pw_impl_device_destroy(device);
