@@ -109,10 +109,10 @@ static char *find_module(const char *path, const char *name, int level)
 }
 
 static int
-global_bind(void *_data, struct pw_impl_client *client, uint32_t permissions,
+global_bind(void *object, struct pw_impl_client *client, uint32_t permissions,
 		 uint32_t version, uint32_t id)
 {
-	struct pw_impl_module *this = _data;
+	struct pw_impl_module *this = object;
 	struct pw_global *global = this->global;
 	struct pw_resource *resource;
 
@@ -134,9 +134,9 @@ error_resource:
 	return -errno;
 }
 
-static void global_destroy(void *object)
+static void global_destroy(void *data)
 {
-	struct pw_impl_module *module = object;
+	struct pw_impl_module *module = data;
 	spa_hook_remove(&module->global_listener);
 	module->global = NULL;
 	pw_impl_module_destroy(module);
@@ -197,6 +197,7 @@ pw_context_load_module(struct pw_context *context,
 				if (hnd != NULL)
 					break;
 
+				pw_log_debug("open failed: %s", dlerror());
 				free(filename);
 				filename = NULL;
 			}
@@ -238,6 +239,8 @@ pw_context_load_module(struct pw_context *context,
 	filename = NULL;
 	this->info.args = args ? strdup(args) : NULL;
 
+	spa_list_prepend(&context->module_list, &this->link);
+
 	this->global = pw_global_new(context,
 				     PW_TYPE_INTERFACE_Module,
 				     PW_VERSION_MODULE,
@@ -248,8 +251,6 @@ pw_context_load_module(struct pw_context *context,
 	if (this->global == NULL)
 		goto error_no_global;
 
-	spa_list_prepend(&context->module_list, &this->link);
-
 	this->info.id = this->global->id;
 	pw_properties_setf(this->properties, PW_KEY_OBJECT_ID, "%d", this->info.id);
 	pw_properties_setf(this->properties, PW_KEY_OBJECT_SERIAL, "%"PRIu64,
@@ -319,11 +320,12 @@ void pw_impl_module_destroy(struct pw_impl_module *module)
 {
 	struct impl *impl = SPA_CONTAINER_OF(module, struct impl, this);
 
-	pw_log_debug("%p: destroy", module);
+	pw_log_debug("%p: destroy %s", module, module->info.name);
 	pw_impl_module_emit_destroy(module);
 
+	spa_list_remove(&module->link);
+
 	if (module->global) {
-		spa_list_remove(&module->link);
 		spa_hook_remove(&module->global_listener);
 		pw_global_destroy(module->global);
 	}
