@@ -22,24 +22,26 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include <unistd.h>
 #include <limits.h>
 #include <stdio.h>
-#ifndef __FreeBSD__
+#if !defined(__FreeBSD__) && !defined(__MidnightBSD__)
 #include <sys/prctl.h>
 #endif
 #include <pwd.h>
 #include <errno.h>
 #include <dlfcn.h>
+#include <pthread.h>
 
 #include <locale.h>
 #include <libintl.h>
 
+#include <valgrind/valgrind.h>
+
 #include <spa/utils/names.h>
+#include <spa/utils/string.h>
 #include <spa/support/cpu.h>
 #include <spa/support/i18n.h>
 
@@ -50,6 +52,11 @@
 
 #define SUPPORTLIB	"support/libspa-support"
 
+PW_LOG_TOPIC_EXTERN(log_context);
+#define PW_LOG_TOPIC_DEFAULT log_context
+
+static char *prgname;
+
 static struct spa_i18n *_pipewire_i18n = NULL;
 
 struct plugin {
@@ -77,15 +84,19 @@ struct support {
 	char **categories;
 	const char *plugin_dir;
 	const char *support_lib;
-	struct registry *registry;
+	struct registry registry;
 	char *i18n_domain;
 	struct spa_interface i18n_iface;
 	struct spa_support support[MAX_SUPPORT];
 	uint32_t n_support;
+	uint32_t init_count;
 	unsigned int in_valgrind:1;
+	unsigned int no_color:1;
+	unsigned int no_config:1;
 };
 
-static struct registry global_registry;
+static pthread_mutex_t init_lock = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t support_lock = PTHREAD_MUTEX_INITIALIZER;
 static struct support global_support;
 
 static struct plugin *
@@ -93,7 +104,7 @@ find_plugin(struct registry *registry, const char *filename)
 {
 	struct plugin *p;
 	spa_list_for_each(p, &registry->plugins, link) {
-		if (!strcmp(p->filename, filename))
+		if (spa_streq(p->filename, filename))
 			return p;
 	}
 	return NULL;
@@ -101,21 +112,18 @@ find_plugin(struct registry *registry, const char *filename)
 
 static struct plugin *
 open_plugin(struct registry *registry,
-	    const char *path, const char *lib)
+	    const char *path, size_t len, const char *lib)
 {
 	struct plugin *plugin;
-	char *filename;
+	char filename[PATH_MAX];
 	void *hnd;
 	spa_handle_factory_enum_func_t enum_func;
 	int res;
 
-        if ((filename = spa_aprintf("%s/%s.so", path, lib)) == NULL) {
-		res = -errno;
+        if ((res = spa_scnprintf(filename, sizeof(filename), "%.*s/%s.so", (int)len, path, lib)) < 0)
 		goto error_out;
-	}
 
 	if ((plugin = find_plugin(registry, filename)) != NULL) {
-		free(filename);
 		plugin->ref++;
 		return plugin;
 	}
@@ -123,7 +131,7 @@ open_plugin(struct registry *registry,
         if ((hnd = dlopen(filename, RTLD_NOW)) == NULL) {
 		res = -ENOENT;
 		pw_log_debug("can't load %s: %s", filename, dlerror());
-		goto error_free_filename;
+		goto error_out;
         }
         if ((enum_func = dlsym(hnd, SPA_HANDLE_FACTORY_ENUM_FUNC_NAME)) == NULL) {
 		res = -ENOSYS;
@@ -138,7 +146,7 @@ open_plugin(struct registry *registry,
 
 	pw_log_debug("loaded plugin:'%s'", filename);
 	plugin->ref = 1;
-	plugin->filename = filename;
+	plugin->filename = strdup(filename);
 	plugin->hnd = hnd;
 	plugin->enum_func = enum_func;
 	spa_list_init(&plugin->handles);
@@ -149,8 +157,6 @@ open_plugin(struct registry *registry,
 
 error_dlclose:
 	dlclose(hnd);
-error_free_filename:
-        free(filename);
 error_out:
 	errno = -res;
 	return NULL;
@@ -186,7 +192,7 @@ static const struct spa_handle_factory *find_factory(struct plugin *plugin, cons
 					factory->version);
 			continue;
 		}
-                if (strcmp(factory->name, factory_name) == 0)
+                if (spa_streq(factory->name, factory_name))
                         return factory;
 	}
 	res = -ENOENT;
@@ -201,29 +207,15 @@ static void unref_handle(struct handle *handle)
 	if (--handle->ref == 0) {
 		spa_list_remove(&handle->link);
 		pw_log_debug("clear handle '%s'", handle->factory_name);
+		pthread_mutex_unlock(&support_lock);
 		spa_handle_clear(&handle->handle);
+		pthread_mutex_lock(&support_lock);
 		unref_plugin(handle->plugin);
 		free(handle->factory_name);
 		free(handle);
 	}
 }
 
-static void configure_debug(struct support *support, const char *str)
-{
-	char **level;
-	int n_tokens;
-
-	level = pw_split_strv(str, ":", INT_MAX, &n_tokens);
-	if (n_tokens > 0)
-		pw_log_set_level(atoi(level[0]));
-
-	if (n_tokens > 1)
-		support->categories = pw_split_strv(level[1], ",", INT_MAX, &n_tokens);
-
-	if (level)
-		pw_free_strv(level);
-}
-
 SPA_EXPORT
 uint32_t pw_get_support(struct spa_support *support, uint32_t max_support)
 {
@@ -233,8 +225,7 @@ uint32_t pw_get_support(struct spa_support *support, uint32_t max_support)
 	return n;
 }
 
-SPA_EXPORT
-struct spa_handle *pw_load_spa_handle(const char *lib,
+static struct spa_handle *load_spa_handle(const char *lib,
 		const char *factory_name,
 		const struct spa_dict *info,
 		uint32_t n_support,
@@ -244,7 +235,9 @@ struct spa_handle *pw_load_spa_handle(const char *lib,
 	struct plugin *plugin;
 	struct handle *handle;
 	const struct spa_handle_factory *factory;
+	const char *state = NULL, *p;
 	int res;
+	size_t len;
 
 	if (factory_name == NULL) {
 		res = -EINVAL;
@@ -256,10 +249,22 @@ struct spa_handle *pw_load_spa_handle(const char *lib,
 
 	pw_log_debug("load lib:'%s' factory-name:'%s'", lib, factory_name);
 
-	if ((plugin = open_plugin(sup->registry, sup->plugin_dir, lib)) == NULL) {
-		res = -errno;
+	plugin = NULL;
+	res = -ENOENT;
+
+	if (sup->plugin_dir == NULL) {
+		pw_log_error("load lib: plugin directory undefined, set SPA_PLUGIN_DIR");
 		goto error_out;
 	}
+	while ((p = pw_split_walk(sup->plugin_dir, ":", &len, &state))) {
+		if ((plugin = open_plugin(&sup->registry, p, len, lib)) != NULL)
+			break;
+		res = -errno;
+	}
+	if (plugin == NULL)
+		goto error_out;
+
+	pthread_mutex_unlock(&support_lock);
 
 	factory = find_factory(plugin, factory_name);
 	if (factory == NULL) {
@@ -281,6 +286,7 @@ struct spa_handle *pw_load_spa_handle(const char *lib,
 		goto error_free_handle;
 	}
 
+	pthread_mutex_lock(&support_lock);
 	handle->ref = 1;
 	handle->plugin = plugin;
 	handle->factory_name = strdup(factory_name);
@@ -291,15 +297,30 @@ struct spa_handle *pw_load_spa_handle(const char *lib,
 error_free_handle:
 	free(handle);
 error_unref_plugin:
+	pthread_mutex_lock(&support_lock);
 	unref_plugin(plugin);
 error_out:
 	errno = -res;
 	return NULL;
 }
 
+SPA_EXPORT
+struct spa_handle *pw_load_spa_handle(const char *lib,
+		const char *factory_name,
+		const struct spa_dict *info,
+		uint32_t n_support,
+		const struct spa_support support[])
+{
+	struct spa_handle *handle;
+	pthread_mutex_lock(&support_lock);
+	handle = load_spa_handle(lib, factory_name, info, n_support, support);
+	pthread_mutex_unlock(&support_lock);
+	return handle;
+}
+
 static struct handle *find_handle(struct spa_handle *handle)
 {
-	struct registry *registry = global_support.registry;
+	struct registry *registry = &global_support.registry;
 	struct plugin *p;
 	struct handle *h;
 
@@ -316,13 +337,16 @@ SPA_EXPORT
 int pw_unload_spa_handle(struct spa_handle *handle)
 {
 	struct handle *h;
+	int res = 0;
 
+	pthread_mutex_lock(&support_lock);
 	if ((h = find_handle(handle)) == NULL)
-		return -ENOENT;
+		res = -ENOENT;
+	else
+		unref_handle(h);
+	pthread_mutex_unlock(&support_lock);
 
-	unref_handle(h);
-
-	return 0;
+	return res;
 }
 
 static void *add_interface(struct support *support,
@@ -334,17 +358,24 @@ static void *add_interface(struct support *support,
 	void *iface = NULL;
 	int res = -ENOENT;
 
-	handle = pw_load_spa_handle(support->support_lib,
+	handle = load_spa_handle(support->support_lib,
 			factory_name, info,
 			support->n_support, support->support);
+	if (handle == NULL)
+		return NULL;
 
-	if (handle == NULL ||
-	    (res = spa_handle_get_interface(handle, type, &iface)) < 0) {
-			pw_log_error("can't get %s interface %d", type, res);
-	} else {
-		support->support[support->n_support++] =
-			SPA_SUPPORT_INIT(type, iface);
+	pthread_mutex_unlock(&support_lock);
+	res = spa_handle_get_interface(handle, type, &iface);
+	pthread_mutex_lock(&support_lock);
+
+	if (res < 0 || iface == NULL) {
+		pw_log_error("can't get %s interface %d: %s", type, res,
+				spa_strerror(res));
+		return NULL;
 	}
+
+	support->support[support->n_support++] =
+		SPA_SUPPORT_INIT(type, iface);
 	return iface;
 }
 
@@ -359,6 +390,7 @@ int pw_set_domain(const char *domain)
 		return -errno;
 	return 0;
 }
+
 SPA_EXPORT
 const char *pw_get_domain(void)
 {
@@ -381,11 +413,6 @@ static const char *i18n_ntext(void *object, const char *msgid, const char *msgid
 
 static void init_i18n(struct support *support)
 {
-	/* Load locale from the environment. */
-	setlocale(LC_ALL, "");
-	/* Set LC_NUMERIC to C so that floating point strings are consistently
-	 * formatted and parsed across locales. */
-	setlocale(LC_NUMERIC, "C");
 	bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
 	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
 	pw_set_domain(GETTEXT_PACKAGE);
@@ -393,11 +420,12 @@ static void init_i18n(struct support *support)
 
 static void *add_i18n(struct support *support)
 {
-	static struct spa_i18n_methods i18n_methods = {
+	static const struct spa_i18n_methods i18n_methods = {
 		SPA_VERSION_I18N_METHODS,
 		.text = i18n_text,
 		.ntext = i18n_ntext,
 	};
+
 	support->i18n_iface = SPA_INTERFACE_INIT(
 			SPA_TYPE_INTERFACE_I18N,
 			SPA_VERSION_I18N,
@@ -422,46 +450,122 @@ const char *pw_ngettext(const char *msgid, const char *msgid_plural, unsigned lo
 }
 
 #ifdef HAVE_SYSTEMD
-static struct spa_log *load_journal_logger(struct support *support)
+static struct spa_log *load_journal_logger(struct support *support,
+					   const struct spa_dict *info)
 {
 	struct spa_handle *handle;
 	void *iface = NULL;
 	int res = -ENOENT;
-	struct spa_dict info;
-	struct spa_dict_item items[1];
-	char level[32];
 	uint32_t i;
 
 	/* is the journal even available? */
 	if (access("/run/systemd/journal/socket", F_OK) != 0)
 		return NULL;
 
-	snprintf(level, sizeof(level), "%d", pw_log_level);
-	items[0] = SPA_DICT_ITEM_INIT(SPA_KEY_LOG_LEVEL, level);
-	info = SPA_DICT_INIT(items, 1);
-
-	handle = pw_load_spa_handle("support/libspa-journal",
-				    SPA_NAME_SUPPORT_LOG, &info,
+	handle = load_spa_handle("support/libspa-journal",
+				    SPA_NAME_SUPPORT_LOG, info,
 				    support->n_support, support->support);
+	if (handle == NULL)
+		return NULL;
 
-	if (handle == NULL ||
-	    (res = spa_handle_get_interface(handle, SPA_TYPE_INTERFACE_Log, &iface)) < 0) {
-			pw_log_error("can't get log interface %d", res);
-	} else {
-		/* look for an existing logger, and
-		 * replace it with the journal logger */
-		for (i = 0; i < support->n_support; i++) {
-			if (strcmp(support->support[i].type, SPA_TYPE_INTERFACE_Log) == 0) {
-				support->support[i].data = iface;
-				break;
-			}
-		}
+	pthread_mutex_unlock(&support_lock);
+	res = spa_handle_get_interface(handle, SPA_TYPE_INTERFACE_Log, &iface);
+	pthread_mutex_lock(&support_lock);
+
+	if (res < 0 || iface == NULL) {
+		pw_log_error("can't get log interface %d: %s", res,
+				spa_strerror(res));
+		return NULL;
 	}
 
+	/* look for an existing logger, and
+	 * replace it with the journal logger */
+	for (i = 0; i < support->n_support; i++) {
+		if (spa_streq(support->support[i].type, SPA_TYPE_INTERFACE_Log)) {
+			support->support[i].data = iface;
+			break;
+		}
+	}
 	return (struct spa_log *) iface;
 }
 #endif
 
+static enum spa_log_level
+parse_log_level(const char *str)
+{
+	enum spa_log_level l = SPA_LOG_LEVEL_NONE;
+	switch(str[0]) {
+		case 'X': l = SPA_LOG_LEVEL_NONE; break;
+		case 'E': l = SPA_LOG_LEVEL_ERROR; break;
+		case 'W': l = SPA_LOG_LEVEL_WARN; break;
+		case 'I': l = SPA_LOG_LEVEL_INFO; break;
+		case 'D': l = SPA_LOG_LEVEL_DEBUG; break;
+		case 'T': l = SPA_LOG_LEVEL_TRACE; break;
+		default:
+			  l = atoi(str);
+			  break;
+	}
+	return l;
+}
+
+static char *
+parse_pw_debug_env(void)
+{
+	const char *str;
+	char **tokens;
+	int n_tokens;
+	size_t slen;
+	char json[1024] = {0};
+	char *pos = json;
+	char *end = pos + sizeof(json) - 1;
+
+	str = getenv("PIPEWIRE_DEBUG");
+
+	if (!str || (slen = strlen(str)) == 0)
+		return NULL;
+
+	/* String format is PIPEWIRE_DEBUG=<glob>:<level>[,<glob>:<level>,...],
+	 * converted into [{ conn.* = 0}, {glob = level}, {glob = level}, ....] ,
+	 * with the connection namespace disabled by default.
+	 */
+	pos += spa_scnprintf(pos, end - pos, "[ { conn.* = %d },", SPA_LOG_LEVEL_NONE);
+
+	/* We only have single-digit log levels, so any single-character
+	 * string is of the form PIPEWIRE_DEBUG=<N> */
+	if (slen == 1) {
+		pw_log_set_level(parse_log_level(str));
+		goto out;
+	}
+
+	tokens = pw_split_strv(str, ",", INT_MAX, &n_tokens);
+	if (n_tokens > 0) {
+		int i;
+		for (i = 0; i < n_tokens; i++) {
+			int n_tok;
+			char **tok;
+			char *pattern;
+			enum spa_log_level lvl;
+
+			tok = pw_split_strv(tokens[i], ":", 2, &n_tok);
+			if (n_tok == 2) {
+				pattern = tok[0];
+				lvl = parse_log_level(tok[1]);
+
+				pos += spa_scnprintf(pos, end - pos, "{ %s = %d },",
+						     pattern, lvl);
+			} else {
+				pw_log_warn("Ignoring invalid format in PIPEWIRE_DEBUG: '%s'\n", tokens[i]);
+			}
+
+			pw_free_strv(tok);
+		}
+	}
+	pw_free_strv(tokens);
+out:
+	pos += spa_scnprintf(pos, end - pos, "]");
+	return strdup(json);
+}
+
 /** Initialize PipeWire
  *
  * \param argc pointer to argc
@@ -470,29 +574,31 @@ static struct spa_log *load_journal_logger(struct support *support)
  * Initialize the PipeWire system, parse and modify any parameters given
  * by \a argc and \a argv and set up debugging.
  *
- * The environment variable \a PIPEWIRE_DEBUG
- *
- * \memberof pw_pipewire
+ * This function can be called multiple times.
  */
 SPA_EXPORT
 void pw_init(int *argc, char **argv[])
 {
 	const char *str;
-	struct spa_dict_item items[5];
+	struct spa_dict_item items[6];
 	uint32_t n_items;
 	struct spa_dict info;
 	struct support *support = &global_support;
 	struct spa_log *log;
 	char level[32];
 
-	if (support->registry != NULL)
-		return;
+	pthread_mutex_lock(&init_lock);
+	if (support->init_count > 0)
+		goto done;
 
-	if ((str = getenv("VALGRIND")))
-		support->in_valgrind = pw_properties_parse_bool(str);
+	pthread_mutex_lock(&support_lock);
+	support->in_valgrind = RUNNING_ON_VALGRIND;
 
-	if ((str = getenv("PIPEWIRE_DEBUG")))
-		configure_debug(support, str);
+	if (getenv("NO_COLOR") != NULL)
+		support->no_color = true;
+
+	if ((str = getenv("PIPEWIRE_NO_CONFIG")) != NULL)
+		support->no_config = pw_properties_parse_bool(str);
 
 	init_i18n(support);
 
@@ -504,19 +610,23 @@ void pw_init(int *argc, char **argv[])
 		str = SUPPORTLIB;
 	support->support_lib = str;
 
-	spa_list_init(&global_registry.plugins);
-	support->registry = &global_registry;
+	spa_list_init(&support->registry.plugins);
 
 	if (pw_log_is_default()) {
+		char *patterns = NULL;
+
 		n_items = 0;
-		if (getenv("NO_COLOR") == NULL)
+		if (!support->no_color)
 			items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_LOG_COLORS, "true");
 		items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_LOG_TIMESTAMP, "true");
-		items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_LOG_LINE, "true");
+		if ((str = getenv("PIPEWIRE_LOG_LINE")) == NULL || spa_atob(str))
+			items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_LOG_LINE, "true");
 		snprintf(level, sizeof(level), "%d", pw_log_level);
 		items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_LOG_LEVEL, level);
 		if ((str = getenv("PIPEWIRE_LOG")) != NULL)
 			items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_LOG_FILE, str);
+		if ((patterns = parse_pw_debug_env()) != NULL)
+			items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_LOG_PATTERNS, patterns);
 		info = SPA_DICT_INIT(items, n_items);
 
 		log = add_interface(support, SPA_NAME_SUPPORT_LOG, SPA_TYPE_INTERFACE_Log, &info);
@@ -524,18 +634,20 @@ void pw_init(int *argc, char **argv[])
 			pw_log_set(log);
 
 #ifdef HAVE_SYSTEMD
-		if ((str = getenv("PIPEWIRE_LOG_SYSTEMD")) == NULL ||
-				strcmp(str, "true") == 0 || atoi(str) != 0) {
-			log = load_journal_logger(support);
+		if ((str = getenv("PIPEWIRE_LOG_SYSTEMD")) == NULL || spa_atob(str)) {
+			log = load_journal_logger(support, &info);
 			if (log)
 				pw_log_set(log);
 		}
 #endif
+		free(patterns);
 	} else {
 		support->support[support->n_support++] =
 			SPA_SUPPORT_INIT(SPA_TYPE_INTERFACE_Log, pw_log_get());
 	}
 
+	pw_log_init();
+
 	n_items = 0;
 	if ((str = getenv("PIPEWIRE_CPU")))
 		items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_CPU_FORCE, str);
@@ -548,15 +660,39 @@ void pw_init(int *argc, char **argv[])
 	add_i18n(support);
 
 	pw_log_info("version %s", pw_get_library_version());
+	pthread_mutex_unlock(&support_lock);
+done:
+	support->init_count++;
+	pthread_mutex_unlock(&init_lock);
 }
 
+/** Deinitialize PipeWire
+ *
+ * Deinitialize the PipeWire system and free up all resources allocated
+ * by pw_init().
+ *
+ * Before 0.3.49 this function can only be called once after which the pipewire
+ * library can not be used again. This is usually called by test programs to
+ * check for memory leaks.
+ *
+ * Since 0.3.49 this function must be paired with an equal amount of pw_init()
+ * calls to deinitialize the PipeWire library. The PipeWire library can be
+ * used again after being deinitialized with a new pw_init() call.
+ */
 SPA_EXPORT
 void pw_deinit(void)
 {
 	struct support *support = &global_support;
-	struct registry *registry = &global_registry;
+	struct registry *registry = &support->registry;
 	struct plugin *p;
 
+	pthread_mutex_lock(&init_lock);
+	if (support->init_count == 0)
+		goto done;
+	if (--support->init_count > 0)
+		goto done;
+
+	pthread_mutex_lock(&support_lock);
 	pw_log_set(NULL);
 	spa_list_consume(p, &registry->plugins, link) {
 		struct handle *h;
@@ -565,11 +701,12 @@ void pw_deinit(void)
 			unref_handle(h);
 		unref_plugin(p);
 	}
-	if (support->categories)
-		pw_free_strv(support->categories);
+	pw_free_strv(support->categories);
 	free(support->i18n_domain);
 	spa_zero(global_support);
-	spa_zero(global_registry);
+	pthread_mutex_unlock(&support_lock);
+done:
+	pthread_mutex_unlock(&init_lock);
 
 }
 
@@ -580,8 +717,6 @@ void pw_deinit(void)
  *
  * Debugging categories can be enabled by using the PIPEWIRE_DEBUG
  * environment variable
- *
- * \memberof pw_pipewire
  */
 SPA_EXPORT
 bool pw_debug_is_category_enabled(const char *name)
@@ -592,13 +727,13 @@ bool pw_debug_is_category_enabled(const char *name)
 		return false;
 
 	for (i = 0; global_support.categories[i]; i++) {
-		if (strcmp(global_support.categories[i], name) == 0)
+		if (spa_streq(global_support.categories[i], name))
 			return true;
 	}
 	return false;
 }
 
-/** Get the application name \memberof pw_pipewire */
+/** Get the application name */
 SPA_EXPORT
 const char *pw_get_application_name(void)
 {
@@ -606,38 +741,52 @@ const char *pw_get_application_name(void)
 	return NULL;
 }
 
-/** Get the program name \memberof pw_pipewire */
+static void init_prgname(void)
+{
+	static char name[PATH_MAX];
+
+	spa_memzero(name, sizeof(name));
+#if defined(__linux__) || defined(__FreeBSD_kernel__) || defined(__MidnightBSD_kernel__)
+	{
+		if (readlink("/proc/self/exe", name, sizeof(name)-1) > 0) {
+			prgname = strrchr(name, '/') + 1;
+			return;
+		}
+	}
+#endif
+#if defined __FreeBSD__ || defined(__MidnightBSD__)
+	{
+		ssize_t len;
+
+		if ((len = readlink("/proc/curproc/file", name, sizeof(name)-1)) > 0) {
+			prgname = strrchr(name, '/') + 1;
+			return;
+		}
+	}
+#endif
+#if !defined(__FreeBSD__) && !defined(__MidnightBSD__)
+	{
+		if (prctl(PR_GET_NAME, (unsigned long) name, 0, 0, 0) == 0) {
+			prgname = name;
+			return;
+		}
+	}
+#endif
+	snprintf(name, sizeof(name), "pid-%d", getpid());
+	prgname = name;
+}
+
+/** Get the program name */
 SPA_EXPORT
 const char *pw_get_prgname(void)
 {
-	static char prgname[PATH_MAX];
-	spa_memzero(prgname, sizeof(prgname));
-#if defined(__linux__) || defined(__FreeBSD_kernel__)
-	{
-		ssize_t len;
-		if ((len = readlink("/proc/self/exe", prgname, sizeof(prgname)-1)) > 0)
-			return strrchr(prgname, '/') + 1;
-	}
-#endif
-#if defined __FreeBSD__
-	{
-		ssize_t len;
-		spa_memzero(prgname, sizeof(prgname));
-		if ((len = readlink("/proc/curproc/file", prgname, sizeof(prgname)-1)) > 0)
-			return strrchr(prgname, '/') + 1;
-	}
-#endif
-#ifndef __FreeBSD__
-	{
-		if (prctl(PR_GET_NAME, (unsigned long) prgname, 0, 0, 0) == 0)
-			return prgname;
-	}
-#endif
-	snprintf(prgname, sizeof(prgname), "pid-%d", getpid());
+	static pthread_once_t prgname_is_initialized = PTHREAD_ONCE_INIT;
+
+	pthread_once(&prgname_is_initialized, init_prgname);
 	return prgname;
 }
 
-/** Get the user name \memberof pw_pipewire */
+/** Get the user name */
 SPA_EXPORT
 const char *pw_get_user_name(void)
 {
@@ -649,7 +798,7 @@ const char *pw_get_user_name(void)
 	return NULL;
 }
 
-/** Get the host name \memberof pw_pipewire */
+/** Get the host name */
 SPA_EXPORT
 const char *pw_get_host_name(void)
 {
@@ -668,11 +817,22 @@ bool pw_in_valgrind(void)
 	return global_support.in_valgrind;
 }
 
+SPA_EXPORT
+bool pw_check_option(const char *option, const char *value)
+{
+	if (spa_streq(option, "in-valgrind"))
+		return global_support.in_valgrind == spa_atob(value);
+	else if (spa_streq(option, "no-color"))
+		return global_support.no_color == spa_atob(value);
+	else if (spa_streq(option, "no-config"))
+		return global_support.no_config == spa_atob(value);
+	return false;
+}
+
 /** Get the client name
  *
  * Make a new PipeWire client name that can be used to construct a remote.
  *
- * \memberof pw_pipewire
  */
 SPA_EXPORT
 const char *pw_get_client_name(void)
@@ -680,18 +840,14 @@ const char *pw_get_client_name(void)
 	const char *cc;
 	static char cname[256];
 
-	if ((cc = pw_get_application_name()))
+	if ((cc = pw_get_application_name()) || (cc = pw_get_prgname()))
 		return cc;
-	else if ((cc = pw_get_prgname()))
-		return cc;
-	else {
-		if (snprintf(cname, sizeof(cname), "pipewire-pid-%zd", (size_t) getpid()) < 0)
-			return NULL;
-		return cname;
-	}
+	else if (snprintf(cname, sizeof(cname), "pipewire-pid-%zd", (size_t) getpid()) < 0)
+		return NULL;
+	return cname;
 }
 
-/** Reverse the direction \memberof pw_pipewire */
+/** Reverse the direction */
 SPA_EXPORT
 enum pw_direction pw_direction_reverse(enum pw_direction direction)
 {
