@@ -32,10 +32,14 @@
 #include <spa/utils/string.h>
 #include <spa/pod/parser.h>
 #include <spa/debug/pod.h>
+#include <spa/param/format-utils.h>
+#include <spa/param/audio/format-utils.h>
+#include <spa/param/video/format-utils.h>
 
 #include <pipewire/impl.h>
 #include <pipewire/extensions/profiler.h>
 
+#define MAX_FORMAT		16
 #define MAX_NAME		128
 
 struct driver {
@@ -59,12 +63,17 @@ struct measurement {
 struct node {
 	struct spa_list link;
 	uint32_t id;
-	char name[MAX_NAME];
+	char name[MAX_NAME+1];
 	struct measurement measurement;
 	struct driver info;
 	struct node *driver;
 	uint32_t errors;
 	int32_t last_error_status;
+	uint32_t generation;
+	char format[MAX_FORMAT+1];
+	struct pw_proxy *proxy;
+	struct spa_hook proxy_listener;
+	struct spa_hook object_listener;
 };
 
 struct data {
@@ -85,6 +94,7 @@ struct data {
 
 	int n_nodes;
 	struct spa_list node_list;
+	uint32_t generation;
 
 	WINDOW *win;
 };
@@ -129,6 +139,98 @@ static struct node *find_node(struct data *d, uint32_t id)
 	return NULL;
 }
 
+static void on_node_removed(void *data)
+{
+	struct node *n = data;
+	pw_proxy_destroy(n->proxy);
+}
+
+static void on_node_destroy(void *data)
+{
+	struct node *n = data;
+	n->proxy = NULL;
+	spa_hook_remove(&n->proxy_listener);
+	spa_hook_remove(&n->object_listener);
+}
+
+static const struct pw_proxy_events proxy_events = {
+	PW_VERSION_PROXY_EVENTS,
+	.removed = on_node_removed,
+	.destroy = on_node_destroy,
+};
+
+static void node_param(void *data, int seq,
+			uint32_t id, uint32_t index, uint32_t next,
+			const struct spa_pod *param)
+{
+	struct node *n = data;
+
+	switch (id) {
+	case SPA_PARAM_Format:
+	{
+		uint32_t media_type, media_subtype;
+
+		spa_format_parse(param, &media_type, &media_subtype);
+
+		switch(media_type) {
+		case SPA_MEDIA_TYPE_audio:
+			switch(media_subtype) {
+			case SPA_MEDIA_SUBTYPE_raw:
+			{
+				struct spa_audio_info_raw info;
+				if (spa_format_audio_raw_parse(param, &info) >= 0) {
+					snprintf(n->format, sizeof(n->format), "%6.6s %d %d",
+						spa_debug_type_find_short_name(spa_type_audio_format, info.format),
+						info.channels, info.rate);
+				}
+				break;
+			}
+			case SPA_MEDIA_SUBTYPE_dsd:
+			{
+				struct spa_audio_info_dsd info;
+				if (spa_format_audio_dsd_parse(param, &info) >= 0) {
+					snprintf(n->format, sizeof(n->format), "DSD%d %d ",
+						8 * info.rate / 44100, info.channels);
+
+				}
+				break;
+			}
+			}
+			break;
+		case SPA_MEDIA_TYPE_video:
+			switch(media_subtype) {
+			case SPA_MEDIA_SUBTYPE_raw:
+			{
+				struct spa_video_info_raw info;
+				if (spa_format_video_raw_parse(param, &info) >= 0) {
+					snprintf(n->format, sizeof(n->format), "%6.6s %dx%d",
+						spa_debug_type_find_short_name(spa_type_video_format, info.format),
+						info.size.width, info.size.height);
+				}
+				break;
+			}
+			}
+			break;
+		case SPA_MEDIA_TYPE_application:
+			switch(media_subtype) {
+			case SPA_MEDIA_SUBTYPE_control:
+				snprintf(n->format, sizeof(n->format), "%s", "CONTROL");
+				break;
+			}
+			break;
+		}
+		break;
+	}
+	default:
+		break;
+	}
+}
+
+static const struct pw_node_events node_events = {
+	PW_VERSION_NODE,
+	.param = node_param,
+};
+
 static struct node *add_node(struct data *d, uint32_t id, const char *name)
 {
 	struct node *n;
@@ -137,11 +239,23 @@ static struct node *add_node(struct data *d, uint32_t id, const char *name)
 		return NULL;
 
 	if (name)
-		strncpy(n->name, name, MAX_NAME-1);
+		strncpy(n->name, name, MAX_NAME);
 	else
 		snprintf(n->name, sizeof(n->name), "%u", id);
 	n->id = id;
 	n->driver = n;
+	n->proxy = pw_registry_bind(d->registry, id, PW_TYPE_INTERFACE_Node, PW_VERSION_NODE, 0);
+	if (n->proxy) {
+		uint32_t ids[1] = { SPA_PARAM_Format };
+
+		pw_proxy_add_listener(n->proxy,
+				&n->proxy_listener, &proxy_events, n);
+		pw_proxy_add_object_listener(n->proxy,
+				&n->object_listener, &node_events, n);
+
+		pw_node_subscribe_params((struct pw_node*)n->proxy,
+						ids, 1);
+	}
 	spa_list_append(&d->node_list, &n->link);
 	d->n_nodes++;
 
@@ -150,6 +264,8 @@ static struct node *add_node(struct data *d, uint32_t id, const char *name)
 
 static void remove_node(struct data *d, struct node *n)
 {
+	if (n->proxy)
+		pw_proxy_destroy(n->proxy);
 	spa_list_remove(&n->link);
 	d->n_nodes--;
 	free(n);
@@ -182,6 +298,7 @@ static int process_driver_block(struct data *d, const struct spa_pod *pod, struc
 	n->measurement = m;
 	n->info = point->info;
 	point->driver = n;
+	n->generation = d->generation;
 
 	if (m.status != 3) {
 		n->errors++;
@@ -216,6 +333,7 @@ static int process_follower_block(struct data *d, const struct spa_pod *pod, str
 
 	n->measurement = m;
 	n->driver = point->driver;
+	n->generation = d->generation;
 	if (m.status != 3) {
 		n->errors++;
 		if (n->last_error_status == -1)
@@ -226,8 +344,12 @@ static int process_follower_block(struct data *d, const struct spa_pod *pod, str
 
 static const char *print_time(char *buf, size_t len, uint64_t val)
 {
-	if (val < 1000000llu)
-		snprintf(buf, len, "%5.1fµs", val/1000.f);
+	if (val == (uint64_t)-1)
+		snprintf(buf, len, "   --- ");
+	else if (val == (uint64_t)-2)
+		snprintf(buf, len, "   +++ ");
+	else if (val < 1000000llu)
+		snprintf(buf, len, "%5.1fus", val/1000.f);
 	else if (val < 1000000000llu)
 		snprintf(buf, len, "%5.1fms", val/1000000.f);
 	else
@@ -235,9 +357,16 @@ static const char *print_time(char *buf, size_t len, uint64_t val)
 	return buf;
 }
 
-static const char *print_perc(char *buf, size_t len, float val, float quantum)
+static const char *print_perc(char *buf, size_t len, uint64_t val, float quantum)
 {
-	snprintf(buf, len, "%5.2f", quantum == 0.0f ? 0.0f : val/quantum);
+	if (val == (uint64_t)-1) {
+		snprintf(buf, len, " --- ");
+	} else if (val == (uint64_t)-2) {
+		snprintf(buf, len, " +++ ");
+	} else {
+		float frac = val / 1000000000.f;
+		snprintf(buf, len, "%5.2f", quantum == 0.0f ? 0.0f : frac/quantum);
+	}
 	return buf;
 }
 
@@ -247,7 +376,8 @@ static void print_node(struct data *d, struct driver *i, struct node *n, int y)
 	char buf2[64];
 	char buf3[64];
 	char buf4[64];
-	float waiting, busy, quantum;
+	uint64_t waiting, busy;
+	float quantum;
 	struct spa_fraction frac;
 
 	if (n->driver == n)
@@ -260,18 +390,30 @@ static void print_node(struct data *d, struct driver *i, struct node *n, int y)
 	else
 		quantum = 0.0;
 
-	waiting = (n->measurement.awake - n->measurement.signal) / 1000000000.f,
-	busy = (n->measurement.finish - n->measurement.awake) / 1000000000.f,
+	if (n->measurement.awake >= n->measurement.signal)
+		waiting = n->measurement.awake - n->measurement.signal;
+	else if (n->measurement.signal > n->measurement.prev_signal)
+		waiting = -2;
+	else
+		waiting = -1;
 
-	mvwprintw(d->win, y, 0, "%s %4.1u %6.1u %6.1u %s %s %s %s  %3.1u  %s%s",
+	if (n->measurement.finish >= n->measurement.awake)
+		busy = n->measurement.finish - n->measurement.awake;
+	else if (n->measurement.awake > n->measurement.prev_signal)
+		busy = -2;
+	else
+		busy = -1;
+
+	mvwprintw(d->win, y, 0, "%s %4.1u %6.1u %6.1u %s %s %s %s  %3.1u %16.16s %s%s",
 			n->measurement.status != 3 ? "!" : " ",
 			n->id,
 			frac.num, frac.denom,
-			print_time(buf1, 64, n->measurement.awake - n->measurement.signal),
-			print_time(buf2, 64, n->measurement.finish - n->measurement.awake),
+			print_time(buf1, 64, waiting),
+			print_time(buf2, 64, busy),
 			print_perc(buf3, 64, waiting, quantum),
 			print_perc(buf4, 64, busy, quantum),
 			i->xrun_count + n->errors,
+			n->measurement.status != 3 ? "" : n->format,
 			n->driver == n ? "" : " + ",
 			n->name);
 }
@@ -283,7 +425,7 @@ static void do_refresh(struct data *d)
 
 	wclear(d->win);
 	wattron(d->win, A_REVERSE);
-	wprintw(d->win, "%-*.*s", COLS, COLS, "S   ID  QUANT   RATE    WAIT    BUSY   W/Q   B/Q  ERR  NAME ");
+	wprintw(d->win, "%-*.*s", COLS, COLS, "S   ID  QUANT   RATE    WAIT    BUSY   W/Q   B/Q  ERR FORMAT           NAME ");
 	wattroff(d->win, A_REVERSE);
 	wprintw(d->win, "\n");
 
@@ -296,6 +438,15 @@ static void do_refresh(struct data *d)
 			break;
 
 		spa_list_for_each(f, &d->node_list, link) {
+			if (d->generation > f->generation + 2) {
+				f->driver = f;
+				spa_zero(f->measurement);
+				spa_zero(f->info);
+				spa_zero(f->format);
+				f->errors = 0;
+				f->last_error_status = 0;
+			}
+
 			if (f->driver != n || f == n)
 				continue;
 
@@ -316,6 +467,7 @@ static void do_refresh(struct data *d)
 static void do_timeout(void *data, uint64_t expirations)
 {
 	struct data *d = data;
+	d->generation++;
 	do_refresh(d);
 }
 
